Name:           openmpi
Version:        1.2.5
Release:        5%{?dist}
Summary:        Open Message Passing Interface

Group:          Development/Libraries
License:        BSD
URL:            http://www.open-mpi.org/
Source0:       	http://www.open-mpi.org/software/ompi/v1.2/downloads/%{name}-%{version}.tar.bz2
Source1:	openmpi.pc.in
Source2:	openmpi.module.in
Source3:	openmpi.sh.in
Source4:	openmpi.csh.in
Patch0:		openmpi-ehca-config.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
BuildRequires:  gcc-g77, libtool, sysfsutils
BuildRequires:  numactl, libibverbs-devel, opensm-devel
Requires:	%{name}-libs = %{version}-%{release}
ExclusiveArch: i386 x86_64 ia64 ppc ppc64

%package libs
Summary:	Libraries used by openmpi programs
Group:		Development/Libraries
Requires:       mpi-selector
Requires(post): /usr/sbin/alternatives, mpi-selector
Requires(preun): mpi-selector

%package devel
Summary:        Development files for openmpi
Group:          Development/Libraries
Requires:       %{name} = %{version}-%{release}
Provides:	mpi-devel

%description
Open MPI is an open source, freely available implementation of both the 
MPI-1 and MPI-2 standards, combining technologies and resources from
several other projects (FT-MPI, LA-MPI, LAM/MPI, and PACX-MPI) in
order to build the best MPI library available.  A completely new MPI-2
compliant implementation, Open MPI offers advantages for system and
software vendors, application developers, and computer science
researchers. For more information, see http://www.open-mpi.org/ .

%description libs
Contains shared libraries used by openmpi applications

%description devel
Contains development headers and libraries for openmpi

# We only compile with gcc, but other people may want other compilers.
# Set the compiler here.
%define opt_cc gcc
# Optional CFLAGS to use with the specific compiler...gcc doesn't need any,
# so uncomment and define to use
#define opt_cc_cflags

# When dealing with multilib installations, aka the ability to run either
# i386 or x86_64 binaries on x86_64 machines, we install the native i386
# openmpi libs/compilers and the native x86_64 libs/compilers.  Obviously,
# on i386 you can only run i386, so you don't really need the -m32 flag
# to gcc in order to force 32 bit mode.  However, since we use the native
# i386 package to support i386 operation on x86_64, and since on x86_64
# the default is x86_64, the i386 package needs to force i386 mode.  This
# is true of all the multilib arches, hence the non-default arch (aka i386
# on x86_64) must force the non-default mode (aka 32 bit compile) in it's
# native-arch package (aka, when built on i386) so that it will work
# properly on the non-native arch as a multilib package (aka i386 installed
# on x86_64).  Just to be safe, we also force the default mode (aka 64 bit)
# in default arch packages (aka, the x86_64 package).  There are, however,
# some arches that don't support forcing *any* mode, those we just leave
# undefined.
%ifarch i386 ppc
%define modeflag -m32
%define mode 32
%endif
%ifarch ia64
%define mode 64
%endif
%ifarch x86_64 ppc64
%define modeflag -m64
%define mode 64
%endif

# We set this to for convenience, since this is the unique dir we use for this
# particular package, version, compiler
%define mpidir %{name}/%{version}-%{opt_cc}

%prep
%setup -q
%patch0 -p1
%ifarch x86_64
XFLAGS="-fPIC"
%endif

./configure --prefix=%{_libdir}/%{mpidir} --with-libnuma=%{_libdir} \
	--with-openib=/usr --enable-mpirun-prefix-by-default \
	CC=%{opt_cc} \
	LDFLAGS='-Wl,-z,noexecstack' \
	CFLAGS="%{?opt_cc_cflags} $RPM_OPT_FLAGS $XFLAGS" \
	CXXFLAGS="$RPM_OPT_FLAGS $XFLAGS" \
	FFLAGS="$RPM_OPT_FLAGS $XFLAGS"

%build
make %{?_smp_mflags}

%install
rm -rf ${RPM_BUILD_ROOT}
make install DESTDIR=${RPM_BUILD_ROOT}
find ${RPM_BUILD_ROOT}%{_libdir}/%{mpidir}/lib -name \*.la | xargs rm

# Make the pkgconfig files
mkdir -p ${RPM_BUILD_ROOT}%{_libdir}/pkgconfig
sed 's#@NAME@#'%{name}'#g;s#@VERSION@#'%{version}'#g;s#@LIBDIR@#'%{_libdir}'#g;s#@CC@#'%{opt_cc}'#g;s#@MPIDIR@#'%{mpidir}'#g;s#@MODEFLAG@#'%{?modeflag}'#g' < %SOURCE1 > ${RPM_BUILD_ROOT}/%{_libdir}/pkgconfig/%{name}-%{version}-%{opt_cc}.pc
sed 's#@LIBDIR@#'%{_libdir}'#g;s#@MPIDIR@#'%{mpidir}'#g;s#@MODEFLAG@#'%{?modeflag}'#g' < %SOURCE2 > ${RPM_BUILD_ROOT}/%{_libdir}/%{mpidir}/lib/%{name}.module

# And do the same for the mpi-selector shell scripts
sed 's#@LIBDIR@#'%{_libdir}/%{mpidir}/lib'#g;s#@BINDIR@#'%{_libdir}/%{mpidir}/bin'#g;s#@MANDIR@#'%{_libdir}/%{mpidir}/share/man'#g' < %SOURCE3 > ${RPM_BUILD_ROOT}%{_libdir}/%{mpidir}/etc/mpivars.sh
sed 's#@LIBDIR@#'%{_libdir}/%{mpidir}/lib'#g;s#@BINDIR@#'%{_libdir}/%{mpidir}/bin'#g;s#@MANDIR@#'%{_libdir}/%{mpidir}/share/man'#g' < %SOURCE4 > ${RPM_BUILD_ROOT}%{_libdir}/%{mpidir}/etc/mpivars.csh

# we need to force the compile mode via the wrapper-data.txt files
# (except on ia64 where the -m64 flag is not allowed by gcc)
%ifnarch ia64
for i in ${RPM_BUILD_ROOT}%{_libdir}/%{mpidir}/share/openmpi/*wrapper-data.txt
do
  sed -e 's#compiler_flags=#compiler_flags='%{?modeflag}' #' < $i > $i.out
  mv $i.out $i
done
%endif

%clean
[ ! -z "${RPM_BUILD_ROOT}" ] && rm -rf ${RPM_BUILD_ROOT}

%post libs
# In order to work when upgrading from older packages that used alternatives
# instead, we will need to forcably remove all the old incarnations of the
# alternatives support, and then install our mpi-selector support.

# We never used anything other than orterun for the mpi-run link
/usr/sbin/alternatives --remove mpi-run /usr/bin/orterun >/dev/null 2>&1

# but, we used opal_wrapper, and then later opal_wrapper-version-gcc-mode for
# mpicc....this little grep/awk thing catches all of them that might still
# be installed
/usr/sbin/alternatives --display mpicc | grep priority | grep opal_wrapper | awk '{ system("/usr/sbin/alternatives --remove mpicc "$1) }'

# ditto for possible variants of mpilibs mpilibs%{mode}, get them all
for i in mpilibs mpilibs%{mode}; do /usr/sbin/alternatives --display $i | grep priority | grep openmpi | awk '{ system("/usr/sbin/alternatives --remove '$i' "$1) }'; done

# It is safe to reregister the same setup at every upgrade/reinstall/etc.
mpi-selector --register %{name}-%{version}-%{opt_cc}-%{_arch} \
    	--source-dir %{_libdir}/%{mpidir}/etc --yes

# check to see if this is an upgrade
if [ $1 -gt 1 ]; then
    # it is...now peform the ugliest hack known to man
    FILE=`mktemp /tmp/openmpi-upgrade-version.XXXXXX`
    echo "VERSION=%{version}" > $FILE
fi

%preun libs
if [ $1 -eq 0 ]; then
    # if this is a straight removal, our job is easy...
    mpi-selector --unregister %{name}-%{version}-%{opt_cc}-%{_arch} --yes 2&>/dev/null
else
    # Upgrades, on the other hand, suck.  The problem is that the version of
    # the new and old packages may not be the same, in which case we want
    # to deregister the old version.  However, we can't just always deregister
    # the old version because on an upgrade, the uninstall happens after the
    # new package has been installed, so running a link removal during this
    # script when the versions are the same would remove the links that we
    # create in the %post install script.  So, we need to catch those times
    # when the version changes and remove our old version then, and the
    # rest of the time leave the links alone on upgrade.  In order to do that
    # we have the %post script above write a temp file with the version of
    # the new package in it and we then read that temp file here to figure
    # out what we should do.
    for i in /tmp/openmpi-upgrade-version.*; do
    	# For the security conscious among us, since we accept all possible
	# input files, we can be easily attacked.  But, since we won't know
	# the real file because it's generated in a script environment we
	# don't have access to, we don't really have much choice.  So,
	# we use head | grep to limit what we allow to be sourced out of the
	# file so we aren't such an easy attack target.
    	eval `head -1 $i | grep "^VERSION=[0-9]+\.[0-9]+\.[0-9]+$"`
	if [ "$$VERSION" -ne "%{version}" ]; then
	    mpi-selector --unregister %{name}-%{version}-%{opt_cc}-%{_arch} \
	    	--yes 2&>/dev/null
	fi
	rm $i
    done
fi
# In case any of the above return an error condition, don't ruin the upgrade
# or removal as they will be non-fatal errors
/bin/true

%files
%defattr(-,root,root,-)
%config(noreplace) %{_libdir}/%{mpidir}/etc/openmpi-*
%{_libdir}/%{mpidir}/bin/orte*
%{_libdir}/%{mpidir}/bin/ompi_info
%{_libdir}/%{mpidir}/bin/mpirun
%{_libdir}/%{mpidir}/bin/mpiexec
%{_libdir}/%{mpidir}/share/man/man1/orterun*
%{_libdir}/%{mpidir}/share/man/man1/mpirun*
%{_libdir}/%{mpidir}/share/man/man1/mpiexec*

%files libs
%defattr(-,root,root,-)
%dir %{_libdir}/%{mpidir}
%dir %{_libdir}/%{mpidir}/lib
%{_libdir}/%{mpidir}/etc/mpivars.*
%{_libdir}/%{mpidir}/lib/*.so.*
%{_libdir}/%{mpidir}/lib/%{name}/*.so
%dir %{_libdir}/%{mpidir}/share/openmpi
%{_libdir}/%{mpidir}/share/openmpi/*
%exclude %{_libdir}/%{mpidir}/share/openmpi/*-wrapper-data.txt

%files devel
%defattr(-,root,root,-)
%{_libdir}/%{mpidir}/bin/opal_wrapper
%{_libdir}/%{mpidir}/bin/mpic*
%{_libdir}/%{mpidir}/bin/mpif*
%{_libdir}/%{mpidir}/bin/mpiCC
%{_libdir}/%{mpidir}/share/man/man1/mpic*
%{_libdir}/%{mpidir}/share/man/man1/mpif*
%{_libdir}/%{mpidir}/share/man/man1/mpiC*
%dir %{_libdir}/%{mpidir}/include
%{_libdir}/%{mpidir}/include/*
%dir %{_libdir}/%{mpidir}/share/man/man3
%{_libdir}/%{mpidir}/share/man/man3/*
%{_libdir}/%{mpidir}/share/openmpi/*-wrapper-data.txt
%{_libdir}/%{mpidir}/lib/*.so
%{_libdir}/pkgconfig/%{name}-%{version}-%{opt_cc}.pc
%{_libdir}/%{mpidir}/lib/*.mod
%{_libdir}/%{mpidir}/lib/*.module


%changelog
* Wed Apr 09 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-5
- Fix the openib transport to know about both part ids on newer ehca adapters

* Tue Apr 01 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-4
- Update to use mpi-selector for mpi package runtime priority selection
- Resolves: bz434805

* Thu Feb 21 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-3
- Openmpi had the same bug as lam in regards to alternatives link management.
  Fix the bug by protecting the uninstall scriptlets from running unless we
  are doing an actual erase of the package.

* Sun Jan 27 2008 Doug Ledford <dledford@redhat.com> - 1.2.5-1
- Update to latest release
- Related: bz441828

* Mon Jul 16 2007 Doug Ledford <dledford@redhat.com> - 1.2.3-4
- Fix directory permissions on the base openmpi directory

* Thu Jul 12 2007 Florian La Roche <laroche@redhat.com> - 1.2.3-3
- requires alternatives for various sub-rpms

* Mon Jul 02 2007 Doug Ledford <dledford@redhat.com> - 1.2.3-2
- Fix dangling symlink issue caused by a bad macro usage

* Wed Jun 27 2007 Doug Ledford <dledford@redhat.com> - 1.2.3-1
- Update to latest upstream version
- Compile against updated openib libraries
- Add udapl as a transport mechanism
- Fix file ownership on -libs package
- Take a swing at solving the multi-install compatibility issues

* Mon Feb 19 2007 Doug Ledford <dledford@redhat.com> - 1.1.1-7
- Bump version to be at least as high as the RHEL4U5 openmpi
- Integrate fixes made in RHEL4 openmpi into RHEL5 (fix a multilib conflict
  for the openmpi.module file by moving from _datadir to _libdir, make sure
  all sed replacements have the g flag so they replace all instances of
  the marker per line, not just the first, and add a %defattr tag to the
  files section of the -libs package to avoid install errors about
  brewbuilder not being a user or group)

* Wed Jan 17 2007 Doug Ledford <dledford@redhat.com> - 1.1.1-5
- Remove the FORTIFY_SOURCE and stack protect options

* Fri Oct 20 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-4
- Bump and build against the final openib-1.1 package

* Wed Oct 18 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-3
- Fix an snprintf length bug in opal/util/cmd_line.c
- RESOLVES: rhbz#210714

* Wed Oct 18 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-2
- Bump and build against openib-1.1-0.pre1.1 instead of 1.0

* Tue Oct 17 2006 Doug Ledford <dledford@redhat.com> - 1.1.1-1
- Update to upstream 1.1.1 version

* Fri Oct 13 2006 Doug Ledford <dledford@redhat.com> - 1.1-7
- ia64 can't take -m64 on the gcc command line, so don't set it there

* Wed Oct 11 2006 Doug Ledford <dledford@redhat.com> - 1.1-6
- Bump rev to match fc6 rev
- Fixup some issue with alternatives support
- Split the 32bit and 64bit libs ld.so.conf.d files into two files so
  multilib or single lib installs both work properly
- Put libs into their own package
- Add symlinks to /usr/share/openmpi/bin%{mode} so that opal_wrapper-%{mode}
  can be called even if it isn't the currently selected default method in
  the alternatives setup (opal_wrapper needs to be called by mpicc, mpic++,
  etc. in order to determine compile mode from argv[0]).

* Sun Aug 27 2006 Doug Ledford <dledford@redhat.com> - 1.1-4
- Make sure the post/preun scripts only add/remove alternatives on initial
  install and final removal, otherwise don't touch.

* Fri Aug 25 2006 Doug Ledford <dledford@redhat.com> - 1.1-3
- Don't ghost the mpi.conf file as that means it will get removed when
  you remove 1 out of a number of alternatives based packages
- Put the .mod file in -devel

* Mon Aug  7 2006 Doug Ledford <dledford@redhat.com> - 1.1-2
- Various lint cleanups
- Switch to using the standard alternatives mechanism instead of a home
  grown one

* Wed Aug  2 2006 Doug Ledford <dledford@redhat.com> - 1.1-1
- Upgrade to 1.1
- Build with Infiniband support via openib

* Mon Jun 12 2006 Jason Vas Dias <jvdias@redhat.com> - 1.0.2-1
- Upgrade to 1.0.2

* Wed Feb 15 2006 Jason Vas Dias <jvdias@redhat.com> - 1.0.1-1
- Import into Fedora Core
- Resolve LAM clashes 

* Wed Jan 25 2006 Orion Poplawski <orion@cora.nwra.com> - 1.0.1-2
- Use configure options to install includes and libraries
- Add ld.so.conf.d file to find libraries
- Add -fPIC for x86_64

* Tue Jan 24 2006 Orion Poplawski <orion@cora.nwra.com> - 1.0.1-1
- 1.0.1
- Use alternatives

* Sat Nov 19 2005 Ed Hill <ed@eh3.com> - 1.0-2
- fix lam conflicts

* Fri Nov 18 2005 Ed Hill <ed@eh3.com> - 1.0-1
- initial specfile created

